/******************************************************************************/
/*Files to Include                                                            */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#elif defined(__18CXX)
    #include <p18cxxx.h>    /* C18 General Include File */
#endif

#if defined(__XC) || defined(HI_TECH_C)

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
#include "user.h"

#endif

//static uint8_t      FF_SCAN;

extern WORD Timer1, Timer2;	/*  */

static int      Sw1Last = 1;
static unsigned int      BlinkTmrCount;
static int      BlinkCount;
static int      BlinkOnOff;
static int      BlinkOnOffSave;
static int      BlinkStart;

/******************************************************************************/
/* Interrupt Routines                                                         */
/******************************************************************************/

/* High-priority service */

#if defined(__XC) || defined(HI_TECH_C)
void interrupt high_isr(void)
#elif defined (__18CXX)
#pragma code high_isr=0x08
#pragma interrupt high_isr
void high_isr(void)
#else
#error "Invalid compiler selection for implemented ISR routines"
#endif

{
    if (TMR0IF == 1){
	WORD n;

	n = Timer1;
	if (n) Timer1 = --n;
	n = Timer2;
	if (n) Timer2 = --n;

        if (CHATTERING_TMR > 0){
            CHATTERING_TMR++;
            if (CHATTERING_TMR == 10){
                CHATTERING_TMR = 0;
            }
        } else {
            uint16_t    sw1tmp;

            sw1tmp = SW1;
            if (Sw1Last == sw1tmp){
                if (sw1tmp == 0){   // press continues
                    SW_CONTINUE_COUNT++;
                } else {
                    SW_CONTINUE_COUNT = 0;
                }
            } else {    // press started or released
                CHATTERING_TMR = 1;     // chattering timer start
                if (sw1tmp == 1){       // released
                    STATE_SW1 = 1;      // SW1 is pressed
                }
            }
            Sw1Last = sw1tmp;
            /*
            if (SW1 == 0 && STATE_SW1 == 0){
                STATE_SW1 = 1;
                CHATTERING_TMR = 1;
            }
            */

            if (SW2 == 0 && STATE_SW2 == 0){
                STATE_SW2 = 1;          // SW2 is pressed
                CHATTERING_TMR = 1;     // chattering timer start
            }
        }
        
        if (LED_BLINK_MODE){
            if (BlinkStart == 0){
                BlinkOnOffSave = LED;
                BlinkStart = 1;
                BlinkTmrCount = 0;
                BlinkOnOff = 0;
                BlinkCount = 0;
            } else {
                BlinkTmrCount++;
                if (BlinkTmrCount == 15){
                    BlinkOnOff = 1 - BlinkOnOff;
                    if (BlinkOnOff == 1){
                        if (BlinkCount == LED_BLINK_MODE){
                            BlinkOnOff = BlinkOnOffSave;
                            LED_BLINK_MODE = 0;
                            BlinkStart = 0;
                        } else {
                            BlinkCount++;
                        }
                    }
                    BlinkTmrCount = 0;
                }
            }
            LED = BlinkOnOff;
        }

        TMR0H = TMR0REG_START_H;
        TMR0L = TMR0REG_START_L;
        TMR0IF = 0;
    }
      /* This code stub shows general interrupt handling.  Note that these
      conditional statements are not handled within 3 seperate if blocks.
      Do not use a seperate if block for each interrupt flag to avoid run
      time errors. */

#if 0
    
      /* TODO Add High Priority interrupt routine code here. */

      /* Determine which flag generated the interrupt */
      if(<Interrupt Flag 1>)
      {
          <Interrupt Flag 1=0>; /* Clear Interrupt Flag 1 */
      }
      else if (<Interrupt Flag 2>)
      {
          <Interrupt Flag 2=0>; /* Clear Interrupt Flag 2 */
      }
      else
      {
          /* Unhandled interrupts */
      }

#endif

}

/* Low-priority interrupt routine */
#if defined(__XC) || defined(HI_TECH_C)
void low_priority interrupt low_isr(void)
#elif defined (__18CXX)
#pragma code low_isr=0x18
#pragma interruptlow low_isr
void low_isr(void)
#else
#error "Invalid compiler selection for implemented ISR routines"
#endif
{

      /* This code stub shows general interrupt handling.  Note that these
      conditional statements are not handled within 3 seperate if blocks.
      Do not use a seperate if block for each interrupt flag to avoid run
      time errors. */

#if 0

      /* TODO Add Low Priority interrupt routine code here. */

      /* Determine which flag generated the interrupt */
      if(<Interrupt Flag 1>)
      {
          <Interrupt Flag 1=0>; /* Clear Interrupt Flag 1 */
      }
      else if (<Interrupt Flag 2>)
      {
          <Interrupt Flag 2=0>; /* Clear Interrupt Flag 2 */
      }
      else
      {
          /* Unhandled interrupts */
      }

#endif

}
