/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#elif defined(__18CXX)
    #include <p18cxxx.h>    /* C18 General Include File */
#endif

#if defined(__XC) || defined(HI_TECH_C)

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#endif

#include "user.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

/* <Initialize variables in user.h and insert code for user algorithms.> */

// MP3_03
void InitApp(void)
{
    /* TODO Initialize User Ports/Peripherals/Project here */
    ADCON1bits.PCFG = 0x0F;     // AN0-12 : All digital I/O
    CCP1CONbits.CCP1M = 0;      // CCP1 disabled
    CCP2CONbits.CCP2M = 0;      // CCP2 disabled
    UCONbits.USBEN = 0;         // USB module disabled
    UCFGbits.UTRDIS = 1;        // On chip tranciever disabled
    T1CONbits.T1OSCEN = 0;      // Timer1 oscillator shut off
    RCSTAbits.SPEN = 0;         // Serial port diabled

    /* Setup analog functionality and port direction */
    // VS1033
    TRISB2 = 0;
    TRISB3 = 0;
    TRISB4 = 0;
    TRISC6 = 0;
    TRISC2 = 0;
    TRISB5 = 0;

    // SW
    TRISA0 = 1;     // input

    // LED
    TRISA3 = 0;

    // LCD

    nRBPU = 0;      // PORTB pullup enable
    
    /* Initialize peripherals */

    /* Configure the IPEN bit (1=on) in RCON to turn on/off int priorities */

    /* Enable interrupts */
    T0CONbits.T08BIT = 0;       // 16bit timer
    T0CONbits.T0CS = 0;         // internal instruction clock cycle
    T0CONbits.PSA = 0;          // Timer0 prescaler is assigned
    T0CONbits.T0PS = 7;         // 1:256 Prescale value
    INTCONbits.GIE = 1;         // Enables all unmasked interrupts
    INTCONbits.TMR0IE = 0;      // Disables the TMR0 overflow interrupt
    TMR0H = TMR0REG_START_H;
    TMR0L = TMR0REG_START_L;
}

void wait_ms(unsigned int ms){
    for (int i=0; i<ms; i++){
        __delay_ms(1);      // pic.h(?) (Unable to resolve mark on MPLABX editer, but compiles OK)
    }
}

/*
 * Write to DATA EEPROM
 */
void write_to_eeprom(unsigned char address, unsigned char data){
    GIE = 0;

    WREN = 1;
    EEADR = address;
    EEDATA = data;
    EECON2 = 0x55;
    EECON2 = 0xaa;
    WR = 1;
    while (EEIF != 1){
    }
    WREN = 0;
    EEIF = 0;

    GIE = 1;
}

/*
 *  Read from DATA EEPROM
 */
unsigned char read_from_eeprom(unsigned char address){
    unsigned char val = 0;

//    GIE = 0;
    EEADR = address;
    EEPGD = 0;
    RD = 1;
    _delay(1);
//    GIE = 1;
    val = EEDATA;
    return val;
}

unsigned short read_from_eeprom_uint16(unsigned char address){
    unsigned short    h;
    h = read_from_eeprom(address + 1);
    h <<= 8;
    h |= read_from_eeprom(address);

    return h;
}

void write_to_eeprom_uint16(unsigned char address, unsigned short data){
    write_to_eeprom(address, data & 0xFF);
    write_to_eeprom(address + 1, (data >> 8) & 0xFF);
}

